@@ -1,6 +1,6 @@
 ext_to_command = {
     ".js": "node",
     ".py": "python",
-    ".out": "./agent.out",
+    ".out": "./",
     ".java": "java",
 }
